--[[ a simple scene manger wrote for this example ]]

--actions is the cutscene manager
require "actions"

scene = {
    actions = {}, -- the actions table 
}

scene._index = scene


function scene:start()
    
    --creating an action is just adding it to the table, 
    --so to write faster I use a local function that just does that and I called "a"
    local a = function(...) table.insert(self.actions, action:new(...)) end
    
    --first let's add the npcs, this part is usually handled from stage when loading a Tiled map.
    --but you can add whatever you want in a scene too, like new npcs.
    
    self.npcs = {}
    
    local mage = lovease.new("mage.aseprite",3)
    table.insert(self.npcs, mage)
    mage.x = 200
    mage.y =200
    
    local knight = lovease.new("knight.aseprite",3)
    table.insert(self.npcs, knight)
    knight.x = 400
    knight.y =200 
    
    local mage2 = lovease.new("mage.aseprite",3)
    table.insert(self.npcs, mage2)
    mage2.x = 200
    mage2.y = 250
    mage2.alpha = 0  -- let's set him invisible to make him appear later
    
    
--- now we can star setting up the scene


--check the actions to see how they work

a("animate", mage,"idleright",true,false)

a("animate", mage,"idleright",true,false)


text=[[So this is a simple action sequence. Press E to advance]]
a("say",text,mage,true)


a("animate",knight,"odessa",true)

text=[[Is simple to que events and create long complex cutscenes]]
a("say",text,knight,true)

text=[[Like the fall of Thornbolt]]
a("say",text,knight,true)

a("animate",mage,"worriedright",true)

text=[[Why I must die ?]]
a("say",text,mage,true)


--create new target position for movement
local position1 = {x = 100, y = 300}
a("moveto", mage, position1, true )


a("animate",knight,"idleleft",true,false)

text=[[Once you have setup all the actions creating the scene is a breeze.]]
a("say",text,knight,true)


a("animate",mage,"sweatright",true,false)

text=[[I still don't want to die..]]
a("say",text,mage,true)

a("hide",mage2,1,true)

a("moveto",mage2, knight,true)

local position2 = {}
position2.x = knight.x + 50
position2.y = knight.y 

text=[[Hey you are      covering me!]]
a("say",text,knight,true)

a("moveto",mage2, position2,true)

a("animate",mage2,"lolleft",2,true)

text=[[Sorry.]]
a("say",text,mage2,true)

a("animate",mage2,"castingleft",true,true)

text=[[Anyway you have to go now.]]
a("say",text,mage2,true)

a("animate",knight,"worriedright",false,true)

text=[[What ?]]
a("say",text,knight,true)

a("animate",mage2,"castleft",false,true)

a("playsfx","magic.ogg")

a("animate",mage,"idleright",true,false)


a("hide",knight,-2,false)
text=[[noooooooooooooooooo....]]
a("say",text,knight,true)

text=[[What a relief]]
a("say",text,mage,true)

a("animate",knight,"idledown",false)

local position3 = {}
position3.x = position2.x 
position3.y = position2.y + 50

a("moveto",mage2,position3,true)

text=[[So, if you have few cutscene just draw them onscreen]]
a("say",text,mage2,true)


text=[[But if you have many and complex then a cutscene manager is the way to go.]]
a("say",text,mage2,true)

text=[[It takes a lil more to code but once done you are the king of the cinematics!]]
a("say",text,mage2,true)


a("hide",knight,1,true)
text=[[Feel free to use this code as base or inspiration for your own.]]
a("say",text,knight,true)


local position4 = {}
position4.x = knight.x -50
position4.y = knight.y + 50

a("moveto",mage,position4,100,true)

a("animate",mage,"smileright",false)
a("animate",mage2,"smile",false)
a("animate",knight,"smiledown",false)

text=[[Bye Bye!]]
a("say",text,knight,true)


a("fadeout",1,2,true)

end


--just a simple key handler for an action key "E" that the action can check. Is used in the say action.
function scene:keyreleased(dt)
    if self.actions then 
        sendkeyreleased(self.actions)
    end
    
end

function scene:update(dt)
    
    --if there is something in my action table let's update it
    if self.actions then 
        
        --executeactions is a global function defined in  actions.lua
        --it can be called from anything, stage, npcs, every entity, just pass a action list to it.
        local success = executeactions(dt,self.actions)
        
        --success means all the actions in the table are completed, in this case i wipe the table
        if success then
            self.actions = type(success) == "table" and success or {}
        end
    end    
    
    --now let's update npcs
    for k,v in pairs(self.npcs) do
        v:update(dt)
    end
end


function scene:draw()
    --some actions have a draw funcion
    if self.actions then 
        drawactions(self.actions)
    end   
    
    
    for k,v in pairs(self.npcs) do
        v:draw()
    end
    
    
    
    --curtain is draw as last to cover up everything
    if self.curtain then
        self.curtain:draw()
    end
    
end


